@@ -9,8 +9,13 @@ use async_std::{
     task::{Context, Poll},
 };
 use cap_primitives::{ambient_authority, fs::is_file_read_write, AmbientAuthority};
+use io_lifetimes::AsFilelike;
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsHandle, BorrowedHandle, FromHandle, IntoHandle, OwnedHandle};
 use std::{fmt, pin::Pin};
-use unsafe_io::{AsUnsafeFile, OwnsRaw};
+use unsafe_io::OwnsRaw;
 #[cfg(windows)]
 use {
     async_std::os::windows::io::{AsRawHandle, FromRawHandle, IntoRawHandle, RawHandle},
@@ -83,7 +88,7 @@ impl File {
     /// This corresponds to [`async_std::fs::File::metadata`].
     #[inline]
     pub fn metadata(&self) -> io::Result<Metadata> {
-        let sync = self.std.as_file_view();
+        let sync = self.std.as_filelike_view::<std::fs::File>();
         metadata_from(&*sync)
     }
 
@@ -94,7 +99,7 @@ impl File {
     /// This corresponds to [`async_std::fs::File::set_permissions`].
     #[inline]
     pub async fn set_permissions(&self, perm: Permissions) -> io::Result<()> {
-        let sync = self.std.as_file_view();
+        let sync = self.std.as_filelike_view::<std::fs::File>();
         self.std
             .set_permissions(permissions_into_std(&sync, perm)?)
             .await
@@ -139,6 +144,14 @@ impl FromRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for File {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(fs::File::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawHandle for File {
     #[inline]
@@ -147,6 +160,14 @@ impl FromRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl FromHandle for File {
+    #[inline]
+    fn from_handle(handle: OwnedHandle) -> Self {
+        Self::from_std(fs::File::from_handle(handle), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for File {
     #[inline]
@@ -155,6 +176,14 @@ impl AsRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl AsFd for File {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandle for File {
     #[inline]
@@ -163,6 +192,14 @@ impl AsRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl AsHandle for File {
+    #[inline]
+    fn as_handle(&self) -> BorrowedHandle<'_> {
+        self.std.as_handle()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for File {
     #[inline]
@@ -179,6 +216,14 @@ impl IntoRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for File {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandle for File {
     #[inline]
@@ -187,6 +232,14 @@ impl IntoRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl IntoHandle for File {
+    #[inline]
+    fn into_handle(self) -> OwnedHandle {
+        self.std.into_handle()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for File {
     #[inline]
@@ -340,7 +393,7 @@ impl fmt::Debug for File {
     // Like libstd's version, but doesn't print the path.
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let mut b = f.debug_struct("File");
-        let file = self.std.as_file_view();
+        let file = self.std.as_filelike_view::<std::fs::File>();
         #[cfg(not(windows))]
         b.field("fd", &file.as_raw_fd());
         #[cfg(windows)]
