@@ -8,6 +8,7 @@ use async_std::{
     task::{Context, Poll},
 };
 use cap_primitives::{ambient_authority, AmbientAuthority};
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
 use std::{fmt, pin::Pin};
 use unsafe_io::OwnsRaw;
 
@@ -107,6 +108,13 @@ impl FromRawFd for UnixStream {
     }
 }
 
+impl FromFd for UnixStream {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(unix::net::UnixStream::from_fd(fd), ambient_authority())
+    }
+}
+
 impl AsRawFd for UnixStream {
     #[inline]
     fn as_raw_fd(&self) -> RawFd {
@@ -114,6 +122,13 @@ impl AsRawFd for UnixStream {
     }
 }
 
+impl AsFd for UnixStream {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.std.as_fd()
+    }
+}
+
 impl IntoRawFd for UnixStream {
     #[inline]
     fn into_raw_fd(self) -> RawFd {
@@ -121,6 +136,13 @@ impl IntoRawFd for UnixStream {
     }
 }
 
+impl IntoFd for UnixStream {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 // Safety: `UnixStream` wraps a `net::UnixStream` which owns its handle.
 unsafe impl OwnsRaw for UnixStream {}
 
